package kr.co.vividnext.sodalive.live

import io.reactivex.rxjava3.core.Flowable
import io.reactivex.rxjava3.core.Single
import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.live.reservation.MakeLiveReservationRequest
import kr.co.vividnext.sodalive.live.reservation.MakeLiveReservationResponse
import kr.co.vividnext.sodalive.live.room.CancelLiveRequest
import kr.co.vividnext.sodalive.live.room.EnterOrQuitLiveRoomRequest
import kr.co.vividnext.sodalive.live.room.LiveRoomStatus
import kr.co.vividnext.sodalive.live.room.StartLiveRequest
import kr.co.vividnext.sodalive.live.room.create.CreateLiveRoomResponse
import kr.co.vividnext.sodalive.live.room.create.GetRecentRoomInfoResponse
import kr.co.vividnext.sodalive.live.room.detail.GetRoomDetailResponse
import kr.co.vividnext.sodalive.live.room.tag.GetLiveTagResponse
import okhttp3.MultipartBody
import okhttp3.RequestBody
import retrofit2.http.Body
import retrofit2.http.GET
import retrofit2.http.Header
import retrofit2.http.Multipart
import retrofit2.http.POST
import retrofit2.http.PUT
import retrofit2.http.Part
import retrofit2.http.Path
import retrofit2.http.Query

interface LiveApi {
    @GET("/live/tag")
    fun getTags(
        @Header("Authorization") authHeader: String
    ): Single<ApiResponse<List<GetLiveTagResponse>>>

    @GET("/live/room")
    fun roomList(
        @Query("timezone") timezone: String,
        @Query("dateString") dateString: String?,
        @Query("status") status: LiveRoomStatus,
        @Query("page") page: Int,
        @Query("size") size: Int,
        @Header("Authorization") authHeader: String
    ): Flowable<ApiResponse<List<GetRoomListResponse>>>

    @GET("/live/room/detail/{id}")
    fun getRoomDetail(
        @Path("id") id: Long,
        @Query("timezone") timezone: String,
        @Header("Authorization") authHeader: String
    ): Single<ApiResponse<GetRoomDetailResponse>>

    @GET("/live/room/recent-room-info")
    fun getRecentRoomInfo(
        @Header("Authorization") authHeader: String
    ): Single<ApiResponse<GetRecentRoomInfoResponse>>

    @POST("/live/room")
    @Multipart
    fun createRoom(
        @Part coverImage: MultipartBody.Part?,
        @Part("request") request: RequestBody,
        @Header("Authorization") authHeader: String
    ): Single<ApiResponse<CreateLiveRoomResponse>>

    @PUT("/live/room/start")
    fun startLive(
        @Body request: StartLiveRequest,
        @Header("Authorization") authHeader: String
    ): Single<ApiResponse<Any>>

    @PUT("/live/room/cancel")
    fun cancelLive(
        @Body request: CancelLiveRequest,
        @Header("Authorization") authHeader: String
    ): Single<ApiResponse<Any>>

    @POST("/live/room/enter")
    fun enterRoom(
        @Body request: EnterOrQuitLiveRoomRequest,
        @Header("Authorization") authHeader: String
    ): Single<ApiResponse<Any>>

    @POST("/live/reservation")
    fun makeReservation(
        @Body request: MakeLiveReservationRequest,
        @Header("Authorization") authHeader: String
    ): Single<ApiResponse<MakeLiveReservationResponse>>

    @PUT("/live/room/{id}")
    @Multipart
    fun editLiveRoomInfo(
        @Path("id") id: Long,
        @Part coverImage: MultipartBody.Part?,
        @Part("request") request: RequestBody?,
        @Header("Authorization") authHeader: String
    ): Single<ApiResponse<Any>>
}
